package org.freehep.jas.plugin.plotter;

import java.awt.*;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.*;
import java.awt.event.*;
import java.util.Properties;
import javax.swing.Icon;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.border.AbstractBorder;
import org.freehep.application.PrintHelper;
import org.freehep.application.mdi.ManagedPage;
import org.freehep.application.mdi.PageContext;
import org.freehep.application.studio.Studio;
import org.freehep.graphicsbase.util.export.ExportDialog;
import org.freehep.graphicsbase.util.export.VectorGraphicsTransferable;
import org.freehep.jas.services.PlotFactory;
import org.freehep.jas.services.PlotPage;
import org.freehep.jas.services.PlotRegion;
import org.freehep.jas.services.PlotRegionDropHandler;
import org.freehep.jas.services.Plotter;
import org.freehep.swing.layout.PercentLayout;
import org.freehep.swing.layout.PercentLayout.Constraint;
import org.freehep.swing.popup.HasPopupItems;
import org.freehep.util.commanddispatcher.CommandProcessor;
import org.freehep.util.images.ImageHandler;


/**
 *
 * @author tonyj
 */
class DefaultPage extends JPanel implements PlotPage, HasPopupItems, ManagedPage {
    private Studio studio;
    private Commands commands = new Commands();
    private PageContext context;
    private String title;
    private PlotFactory factory;
    private static final Icon histogramIcon = ImageHandler.getIcon("images/Histogram",DefaultPage.class);
    private DefaultRegion currentRegion;
    private MouseListener ml = new RegionListener();
    private PercentLayout layout;
    private int nColumns = -1;
    private int nRows = -1;
    private int nAddedRegions = 0;

    /** Creates a new instance of DefaultPage */
    public DefaultPage(Studio studio, PlotFactory factory, String title) {
        this.studio = studio;
        setTitle( title );
        this.factory = factory;
        setLayout(layout = new PercentLayout());
        setBackground(Color.white);
        setPreferredSize(new Dimension(600,600));
        createRegion(0,0,1,1);
    }

    protected void setTitle( String title ) {
        this.title = title;
        if ( context != null )
            context.setTitle(title);
    }

    protected String title() {
        return title;
    }

    protected int columns() {
        return nColumns;
    }

    protected int rows() {
        return nRows;
    }

    protected int addedRegions() {
        return nAddedRegions;
    }

    public void clearRegions() {
        removeAll();
        currentRegion = null;
        nColumns = -1;
        nRows = -1;
        nAddedRegions = 0;
    }

    public PlotRegion createRegion(double x, double y, double w, double h) {
        DefaultRegion result = new DefaultRegion(factory);
        result.addMouseListener(ml);
        add(result,new Constraint(x*100,y*100,w*100,h*100));
        setCurrentRegion(result);
        return result;
    }

    public void createRegions(int columns, int rows) {
        clearRegions();
        nColumns = columns;
        nRows = rows;
        if (rows*columns != 0) {
            double pcWidth = 100/columns;
            double pcHeight = 100/rows;

            for (int r=0; r<rows; r++) {
                for (int c=0; c<columns; c++) {
                    DefaultRegion region = new DefaultRegion(factory);
                    region.addMouseListener(ml);
                    add(region,new Constraint(c*pcWidth,r*pcHeight,pcWidth,pcHeight));
                    if (r==0 && c==0) setCurrentRegion(region);
                }
            }
        }
    }


    public PlotRegion currentRegion() {
        return currentRegion;
    }

    public void hidePage() {
        context.close();
    }

    public PlotRegion next() {
        Component[] c = getComponents();
        for (int i=0; i<c.length; i++) {
            if (c[i] == currentRegion) {
                int index = (i+1)%c.length;
                setCurrentRegion((PlotRegion) c[index]);
                return currentRegion;
            }
        }
        return null;
    }

    public int numberOfRegions() {
        return getComponentCount();
    }

    public PlotRegion region(int index) {
        return (PlotRegion) getComponent(index);
    }

    public void setCurrentRegion(PlotRegion reg) {
        if (reg != currentRegion) {
            if (!(reg instanceof DefaultRegion)) throw new IllegalArgumentException();
            if (currentRegion != null) currentRegion.setSelected(false);
            currentRegion = (DefaultRegion) reg;
            if (currentRegion != null) currentRegion.setSelected(true);
        }
    }

    public void showPage() {
        if (context == null) {
            context = studio.getPageManager().openPage(this,title,histogramIcon, "Plot");
        }
        else context.requestShow();
    }
    public JPopupMenu modifyPopupMenu(JPopupMenu menu, Component source, Point p) {
        studio.getXMLMenuBuilder().mergePopupMenu("PlotPagePopupMenu",menu);
        return menu;
    }

    public boolean close() {
        context = null;
        return true;
    }
    public void pageClosed() {
    }
    public void pageDeiconized() {
    }
    public void pageIconized() {
    }
    public void pageSelected() {
        studio.getCommandTargetManager().add(commands);
    }
    public void pageDeselected() {
        studio.getCommandTargetManager().remove(commands);
    }
    public void setPageContext(PageContext context) {
    }

    public Component viewable() {
        return this;
    }

    public PlotRegion addRegion() {
        nAddedRegions++;
        int n = getComponentCount();
        DefaultRegion result = new DefaultRegion(factory);
        result.addMouseListener(ml);
        add(result,new Constraint(0,0,100,100));
        setCurrentRegion(result);
        if      (n < 2) setLayout(2,1);
        else if (n < 3) setLayout(3,1);
        else if (n < 4) setLayout(2,2);
        else if (n < 6) setLayout(3,2);
        else if (n < 9) setLayout(3,3);
        else if (n < 12) setLayout(4,3);
        else if (n < 16) setLayout(4,4);
        else if (n < 20) setLayout(5,4);
        else if (n < 25) setLayout(5,5);
        else if (n < 30) setLayout(6,5);
        else if (n < 36) setLayout(6,6);
        else {
            int nn = 1 + (int) Math.floor(Math.sqrt(n-1));
            setLayout(nn,nn);
        }
        revalidate();
        repaint();
        return result;
    }
    private void setLayout(int columns, int rows) {
        if (rows*columns != 0) {
            double pcWidth = 100/columns;
            double pcHeight = 100/rows;

            Component[] comps = getComponents();
            int i=0;
            for (int r=0; r<rows; r++) {
                for (int c=0; c<columns; c++) {
                    if (i >= comps.length) return;
                    Constraint constraint = layout.getConstraintFor(comps[i++]);
                    constraint.setConstraints(c*pcWidth,r*pcHeight,pcWidth,pcHeight);
                }
            }
         }
   }
   private class RegionListener extends MouseAdapter
   {
      public void mouseClicked(MouseEvent e)
      {
          DefaultRegion reg = (DefaultRegion) e.getSource();
         setCurrentRegion(reg);
      }
   }
   private class Commands extends CommandProcessor
   {
      public void onRegions_1_1()
      {
         createRegions(1,1);
      }
      public void onRegions_2_1()
      {
         createRegions(2,1);
      }
      public void onRegions_1_2()
      {
         createRegions(1,2);
      }
      public void onRegions_2_2()
      {
         createRegions(2,2);
      }
      public void onRegions_3_1()
      {
         createRegions(3,1);
      }
      public void onRegions_1_3()
      {
         createRegions(1,3);
      }
      public void onRegions_3_2()
      {
         createRegions(3,2);
      }
      public void onRegions_2_3()
      {
         createRegions(2,3);
      }
      public void onRegions_3_3()
      {
         createRegions(3,3);
      }
      public void onRegions_4_4()
      {
         createRegions(4,4);
      }
      public void onCopy()
      {
         Clipboard cb = DefaultPage.this.getToolkit().getSystemClipboard();
         VectorGraphicsTransferable t = new VectorGraphicsTransferable(DefaultPage.this);
         cb.setContents(t,t);
      }
      public void onPrintSetup()
      {
         PrintHelper ph = new PrintHelper(DefaultPage.this, studio);
         ph.showOptionsDialog(studio);
      }
      public void onPrint() throws Exception
      {
          PrintHelper ph = new PrintHelper(DefaultPage.this, studio);
          //Temporary fix to JAS-410
          jas.plot.PrintHelper.instance().setPrintingThread(Thread.currentThread());       
          ph.print();
          jas.plot.PrintHelper.instance().setPrintingThread(null);
      }
      public void onPrintPreview() throws Exception
      {
          PrintHelper ph = new PrintHelper(DefaultPage.this, studio);
          //Temporary fix to JAS-410
          jas.plot.PrintHelper.instance().setPrintingThread(Thread.currentThread());       
          ph.printPreview(studio);
          jas.plot.PrintHelper.instance().setPrintingThread(null);
      }
      public void onSaveAs()
      {
         Properties user = studio.getUserProperties();
         String creator = user.getProperty("fullVersion");
         ExportDialog dlg = new ExportDialog(creator,true);
         dlg.setUserProperties(user);
                //Temporary fix to JAS-410
                jas.plot.PrintHelper.instance().setPrintingThread(Thread.currentThread());       
         dlg.showExportDialog(studio, "Save As...",  DefaultPage.this, "plotpage");
                jas.plot.PrintHelper.instance().setPrintingThread(null);       
      }
      public void onClearAllRegions() {
          for ( int i = 0; i < numberOfRegions(); i++ )
              region(i).clear();
      }
   }
}

class DefaultRegion extends JPanel implements PlotRegion, DropTargetListener, HasPopupItems, ActionListener  {
    private RegionBorder border = new RegionBorder();
    private Color activeBorderColor = Color.red;
    private Color defaultBorderColor = Color.green;
    private Color selectedBorderColor = Color.blue;
    private PlotFactory factory;
    private Plotter plotter;
    private PlotRegionDropHandler defaultDropHandler;
    private boolean selected = false;

    DefaultRegion(PlotFactory factory) {
        super(new BorderLayout());
        this.factory = factory;
        setBorder(border);
        border.setColor(defaultBorderColor);
        defaultDropHandler = new DefaultPlotRegionDropHandler(this, factory);
        setOpaque(false);
        DropTarget dt = new DropTarget(this,this);
    }
    void setSelected(boolean selected) {
        this.selected = selected;
        border.setColor(selected ? selectedBorderColor : defaultBorderColor);
        repaint();
    }
    public void dragEnter(DropTargetDragEvent dtde) {
        if (acceptOrReject(dtde)) {
            border.setColor(activeBorderColor);
            repaint();
        }
    }
    public void dragExit(DropTargetEvent dte) {
        border.setColor(selected ? selectedBorderColor : defaultBorderColor);
        repaint();
    }
    boolean acceptOrReject(DropTargetDragEvent e) {
        boolean accept = false;
        DataFlavor[] flavors = e.getCurrentDataFlavors();
        for (int i=0; i<flavors.length; i++) {
            Class k = flavors[i].getRepresentationClass();
            // Test if a plotter is available for this class
            accept = Plotter.class.isAssignableFrom(k) || factory.canCreatePlotterFor(k);
            if (accept) break;
        }

        if (accept) e.acceptDrag(DnDConstants.ACTION_LINK);
        else e.rejectDrag();
        return accept;
    }
    public void dragOver(DropTargetDragEvent dtde) {
    }
    public void drop(DropTargetDropEvent dtde) {
        border.setColor(defaultBorderColor);
        repaint();
        factory.currentPage().setCurrentRegion(this);
        try {
            PlotRegionDropHandler dh = getPlotRegionDropHandler(dtde);
            if (dh == null) defaultDropHandler.drop(dtde);
            else {
                dh.setPlotRegion(this);
                dh.drop(dtde);                
            }
        }
        catch (Throwable x) {
            x.printStackTrace();
            dtde.dropComplete(false);
        }
    }
    public void dropActionChanged(DropTargetDragEvent dtde) {
    }
    
    public PlotRegionDropHandler getPlotRegionDropHandler(DropTargetDropEvent dtde) {
        PlotRegionDropHandler dh = null;
        try {
            Transferable t = dtde.getTransferable();
            DataFlavor[] flavors = t.getTransferDataFlavors();
            for (int i=0; i<flavors.length; i++) {
                Class k = flavors[i].getRepresentationClass();
                if (PlotRegionDropHandler.class.isAssignableFrom(k)) {
                    dh = (PlotRegionDropHandler) t.getTransferData(flavors[i]);
                    break;
                }
            }
        }
        catch (Throwable x) {
            x.printStackTrace();
            dtde.dropComplete(false);
        }
        return dh;        
    }

    public void showPlot(Plotter plotter) {
        removeAll();
        this.plotter = plotter;
        add(plotter.viewable(), BorderLayout.CENTER);
        revalidate();
        repaint();
    }

    public void clear() {
        if ( plotter != null ) {
            plotter.clear();
            plotter = null;
        }
        removeAll();
        revalidate();
        repaint();
    }

    public Plotter currentPlot() {
        return plotter;
    }

    public void actionPerformed(ActionEvent actionEvent) {
       String command = actionEvent.getActionCommand();
       if      (command.equals("clear")) clear();
    }

    public JPopupMenu modifyPopupMenu(JPopupMenu menu, Component source, Point p) {
        JMenuItem item = new JMenuItem("Clear Region");
        item.setActionCommand("clear");
        item.addActionListener(this);
        menu.add(item);
        return menu;
    }

    protected void printBorder(Graphics g)
    {
       // We dont want to print the border, which is just used to show
       // the current selection
    }
    
}
class RegionBorder extends AbstractBorder {
    private static final Insets insets = new Insets(1, 1, 1, 1);
    private Color color;
    void setColor(Color color) {
        this.color = color;
    }
    public void paintBorder(Component c,Graphics g,int x,int y,int width,int height) {
        g.setColor(color != null ? color : c.getBackground());
        g.drawRect(x, y, width-1, height-1);
    }
    public boolean isBorderOpaque() {
        return true;
    }
    public Insets getBorderInsets(Component c) {
        return insets;
    }
}
