package org.freehep.jas.plugin.plotter;

import java.util.ArrayList;
import org.freehep.jas.services.PlotterAdapter;

/**
 *
 * @author The FreeHEP team @ SLAC
 *
 */
public class PlotterAdapterLookup {
    
    private ArrayList adapters = new ArrayList();
    
    public void registerAdapter(PlotterAdapter adapter, Class from, Class to) {
        adapters.add( new AdapterEntry(adapter,from,to) );
    }

    public PlotterAdapter adapter( Class from, Class to ) {
        ArrayList fromAdapters = new ArrayList();
        ArrayList toAdapters = new ArrayList();
        
        for ( int i = 0; i < adapters.size(); i++ ) {
            AdapterEntry entry = (AdapterEntry)adapters.get(i);
            if ( entry.isAdapter( from, to ) ) return entry.adapter();
            if ( entry.isAdapterFrom( from ) )
                fromAdapters.add( entry );
            if ( entry.isAdapterTo(to) )
                toAdapters.add( entry );
        }
        
        if ( fromAdapters.size() == 0 || toAdapters.size() == 0 ) return null;
        
        for ( int i = 0; i < fromAdapters.size(); i++ ) {
            AdapterEntry fromEntry = (AdapterEntry)fromAdapters.get(i);
            for ( int j = 0; j < toAdapters.size(); j++ ) {
                AdapterEntry toEntry = (AdapterEntry)toAdapters.get(j);
                if ( fromEntry.isAdapterTo( toEntry.from() ) ) {
                    PlotterAdapter combinedAdapter = new CombinedAdapter( fromEntry.adapter(), toEntry.adapter() );
//                    registerAdapter( combinedAdapter, fromEntry.from(), toEntry.to() );
                    return combinedAdapter;
                } 
/*                
                else {
                    System.out.println("Looking for an adapter from "+fromEntry.adapter()+" to "+toEntry.adapter());
                    System.out.println("Looking for an adapter from "+fromEntry.to()+" to "+toEntry.from());
                    PlotterAdapter inBetweenAdapter = adapter(fromEntry.to(), toEntry.from());
                    if ( inBetweenAdapter != null ) {
                        PlotterAdapter combinedAdapter = new CombinedAdapter( fromEntry.adapter(), inBetweenAdapter, toEntry.adapter() );
//                        registerAdapter( combinedAdapter, fromEntry.from(), toEntry.to() );
                        return combinedAdapter;
                    }
                }
 */
            }
        }
        return null;
    }
    
    private class AdapterEntry {
        private PlotterAdapter adapter;
        private Class from;
        private Class to;
        private Class inBetween;
        
        public AdapterEntry(PlotterAdapter adapter, Class from, Class to) {
            this.adapter = adapter;
            this.from = from;
            this.to = to;
        }

        public PlotterAdapter adapter() {
            return adapter;
        }
        
        public Class from() {
            return from;
        }
        
        public Class to() {
            return to;
        }
        
        public boolean isAdapter( Class from, Class to ) {
            return isAdapterFrom(from) && isAdapterTo(to);
        }
        
        public boolean isAdapterFrom( Class from ) {
            return from().isAssignableFrom( from );
        }
        
        public boolean isAdapterTo( Class to ) {
            return to.isAssignableFrom( to() );
        }
            
    }
    
    private class CombinedAdapter implements PlotterAdapter {
        
        private PlotterAdapter fromAdapter;
        private PlotterAdapter toAdapter;
        private PlotterAdapter inBetweenAdapter;
        
        public CombinedAdapter( PlotterAdapter fromAdapter, PlotterAdapter toAdapter ) {
            this.fromAdapter = fromAdapter;
            this.toAdapter = toAdapter;
        }

        public CombinedAdapter( PlotterAdapter fromAdapter, PlotterAdapter inBetweenAdapter, PlotterAdapter toAdapter ) {
            this.fromAdapter = fromAdapter;
            this.toAdapter = toAdapter;
            this.inBetweenAdapter = inBetweenAdapter;
        }
        
        public Object adapt( Object obj ) {
            if ( inBetweenAdapter == null ) 
                return toAdapter.adapt( fromAdapter.adapt( obj ) );
            System.out.println("** "+fromAdapter);
            System.out.println("** "+inBetweenAdapter);
            System.out.println("** "+toAdapter);
            
            return toAdapter.adapt( inBetweenAdapter.adapt( fromAdapter.adapt( obj ) ) );
        }
    }
    
}
